@@ -1,8 +1,9 @@
-# Copyright 2001-2004 The Apache Software Foundation
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
@@ -82,6 +83,7 @@ require Exporter;
 @ISA = qw(LWP::UserAgent);
 
 my $UA;
+my $REDIR = $have_lwp ? undef : 1;
 
 sub module {
     my $module = shift;
@@ -106,6 +108,21 @@ sub module2path {
     return $path;
 }
 
+sub module2url {
+    my $module   = shift;
+    my $opt      = shift || {};
+    my $scheme   = $opt->{scheme} || 'http';
+    my $path     = exists $opt->{path} ? $opt->{path} : module2path($module);
+
+    module($module);
+
+    my $config   = Apache::Test::config();
+    my $hostport = hostport($config);
+
+    $path =~ s|^/||;
+    return "$scheme://$hostport/$path";
+}
+
 sub user_agent {
     my $args = {@_};
 
@@ -116,9 +133,19 @@ sub user_agent {
     if (exists $args->{requests_redirectable}) {
         my $redir = $args->{requests_redirectable};
         if (ref $redir and (@$redir > 1 or $redir->[0] ne 'POST')) {
-            $RedirectOK = 1;
+            # Set our internal flag if there's no LWP.
+            $REDIR = $have_lwp ? undef : 1;
+        } elsif ($redir) {
+            if ($have_lwp) {
+                $args->{requests_redirectable} = [ qw/GET HEAD POST/ ];
+                $REDIR = undef;
+            } else {
+                # Set our internal flag.
+                $REDIR = 1;
+            }
         } else {
-            $RedirectOK = 0;
+            # Make sure our internal flag is false if there's no LWP.
+            $REDIR = $have_lwp ? undef : 0;
         }
     }
 
@@ -154,16 +181,19 @@ sub do_request {
 
 sub hostport {
     my $config = shift || Apache::Test::config();
-    local $config->{vars}->{scheme} =
-      $Apache::TestRequest::Scheme || $config->{vars}->{scheme};
+    my $vars = $config->{vars};
+    local $vars->{scheme} =
+        $Apache::TestRequest::Scheme || $vars->{scheme};
     my $hostport = $config->hostport;
 
+    my $default_hostport = join ':', $vars->{servername}, $vars->{port};
     if (my $module = $Apache::TestRequest::Module) {
-        $hostport = $config->{vhosts}->{$module}->{hostport}
-          unless $module eq 'default';
+        $hostport = $module eq 'default'
+            ? $default_hostport
+            : $config->{vhosts}->{$module}->{hostport};
     }
 
-    $hostport;
+    $hostport || $default_hostport;
 }
 
 sub resolve_url {
@@ -192,12 +222,19 @@ sub wanted_args {
     \%wanted_args;
 }
 
-$RedirectOK = 1;
-
 sub redirect_ok {
-    my($self, $request) = @_;
-    return 0 if $request->method eq 'POST';
-    $RedirectOK;
+    my $self = shift;
+    if ($have_lwp) {
+        # Return user setting or let LWP handle it.
+        return $RedirectOK if defined $RedirectOK;
+        return $self->SUPER::redirect_ok(@_);
+    }
+
+    # No LWP. We don't support redirect on POST.
+    return 0 if $self->method eq 'POST';
+    # Return user setting or our internal calculation.
+    return $RedirectOK if defined $RedirectOK;
+    return $REDIR;
 }
 
 my %credentials;
@@ -235,9 +272,7 @@ sub vhost_socket {
     local $Apache::TestRequest::Module = $module if $module;
 
     my $hostport = hostport(Apache::Test::config());
-    die "can't find hostport for '$module',\n",
-        "make sure that vhost_socket() was passed a valid module name"
-            unless defined $hostport;
+
     my($host, $port) = split ':', $hostport;
     my(%args) = (PeerAddr => $host, PeerPort => $port);
 
@@ -322,7 +357,7 @@ sub prepare {
 sub UPLOAD {
     my($url, $pass, $keep) = prepare(@_);
 
-    local $RedirectOK = exists $keep->{redirect_ok} 
+    local $RedirectOK = exists $keep->{redirect_ok}
         ? $keep->{redirect_ok}
         : $RedirectOK;
 
@@ -519,7 +554,11 @@ sub same_interp_tie {
     my($url) = @_;
 
     my $res = GET($url, INTERP_KEY, 'tie');
-
+    unless ($res->code == 200) {
+        die sprintf "failed to init the same_handler data (url=%s). " .
+            "Failed with code=%s, response:\n%s",
+                $url, $res->code, $res->content;
+    }
     my $same_interp = $res->header(INTERP_KEY);
 
     return $same_interp;
@@ -531,7 +570,7 @@ sub same_interp_tie {
 sub same_interp_do {
     my($same_interp, $sub, $url, @args) = @_;
 
-    die "must pass an interpreter id to work with"
+    die "must pass an interpreter id, obtained via same_interp_tie()"
         unless defined $same_interp and $same_interp;
 
     push @args, (INTERP_KEY, $same_interp);
@@ -542,10 +581,18 @@ sub same_interp_do {
     do {
         #loop until we get a response from our interpreter instance
         $res = $sub->($url, @args);
-
-        if ($res and $res->code == 200) {
+        die "no result" unless $res;
+        my $code = $res->code;
+        if ($code == 200) {
             $found_same_interp = $res->header(INTERP_KEY) || '';
         }
+        elsif ($code == 404) {
+            # try again
+        }
+        else {
+            die sprintf "failed to run the request (url=%s):\n" .
+                "code=%s, response:\n%s", $url, $code, $res->content;
+        }
 
         unless ($found_same_interp eq $same_interp) {
             $found_same_interp = '';
@@ -727,22 +774,52 @@ connection is used to process multiple requests (and, according to the
 C<L<LWP::UserAgent|LWP::UserAgent>> documentation, has the effect of
 loading and enabling the new experimental HTTP/1.1 protocol module).
 
-And finally, the semantics of the C<requests_redirectable> parameter
-is different than for C<LWP::UserAgent>: It either follows
-redirects for a request, or it doesn't. Thus
-C<requests_redirectable> is a boolean value instead of the array
-reference that C<LWP::UserAgent> expects. To force
-C<Apache::TestRequest> not to follow redirects in any of its
-convenience functions, pass a false value to C<requests_redirectable>:
+And finally, the semantics of the C<requests_redirectable> parameter is
+different than for C<LWP::UserAgent> in that you can pass it a boolean
+value as well as an array for C<LWP::UserAgent>. To force
+C<Apache::TestRequest> not to follow redirects in any of its convenience
+functions, pass a false value to C<requests_redirectable>:
 
   Apache::TestRequest::user_agent(reset => 1,
                                   requests_redirectable => 0);
 
+If LWP is not installed, then you can still pass in an array reference
+as C<LWP::UserAgent> expects. C<Apache::TestRequest> will examine the
+array and allow redirects if the array contains more than one value or
+if there is only one value and that value is not "POST":
+
+  # Always allow redirection.
+  my $redir = have_lwp() ? [qw(GET HEAD POST)] : 1;
+  Apache::TestRequest::user_agent(reset => 1,
+                                  requests_redirectable => $redir);
+
+But note that redirection will B<not> work with C<POST> unless LWP is
+installed. It's best, therefore, to check C<have_lwp> before running
+tests that rely on a redirection from C<POST>.
+
+Sometimes it is desireable to have C<Apache::TestRequest> remember
+cookies sent by the pages you are testing and send them back to the
+server on subsequent requests. This is especially necessary when
+testing pages whose functionality relies on sessions or the presence
+of preferences stored in cookies.
+
+By default, C<LWP::UserAgent> does B<not> remember cookies between
+requests. You can tell it to remember cookies between request by
+adding:
+
+  Apache::TestRequest::user_agent(cookie_jar => {});
+
+before issuing the requests.
+
+
 =head1 FUNCTIONS
 
 C<Apache::TestRequest> exports a number of functions that will likely
 prove convenient for use in the majority of your request tests.
 
+
+
+
 =head2 Optional Parameters
 
 Each function also takes a number of optional arguments.
@@ -795,6 +872,11 @@ test server via C<UPLOAD()> and its friends.
 Sends a simple GET request to the Apache test server. Returns an
 C<HTTP::Response> object.
 
+You can also supply additional headers to be sent with the request by
+adding their name/value pairs after the C<url> parameter, for example:
+
+  my $res = GET $url, 'Accept-Language' => 'de,en-us,en;q=0.5';
+
 =head3 GET_STR
 
 A shortcut function for C<GET($uri)-E<gt>as_string>.
@@ -1011,6 +1093,70 @@ testing what options the Apache server supports. Consult the HTTPD 1.1
 specification, section 9.2, at
 I<http://www.faqs.org/rfcs/rfc2616.html> for more information.
 
+
+
+
+
+=head2 URL Manipulation Functions
+
+C<Apache::TestRequest> also includes a few helper functions to aid in
+the creation of urls used in the functions above.
+
+
+
+=head3 C<module2path>
+
+  $path = Apache::TestRequest::module2path($module_name);
+
+Convert a module name to a path, safe for use in the various request
+methods above. e.g. C<::> can't be used in URLs on win32. For example:
+
+  $path = Apache::TestRequest::module2path('Foo::Bar');
+
+returns:
+
+  /Foo__Bar
+
+
+
+
+=head3 C<module2url>
+
+  $url = Apache::TestRequest::module2url($module);
+  $url = Apache::TestRequest::module2url($module, \%options);
+
+Convert a module name to a full URL including the current
+configurations C<hostname:port> and sets C<module> accordingly.
+
+  $url = Apache::TestRequest::module2url('Foo::Bar');
+
+returns:
+
+  http://$hostname:$port/Foo__Bar
+
+The default scheme used is C<http>. You can override this by passing
+your preferred scheme into an optional second param. For example:
+
+  $module = 'MyTestModule::TestHandler';
+  $url = Apache::TestRequest::module2url($module, {scheme => 'https'});
+
+returns:
+
+  https://$hostname:$port/MyTestModule__TestHandler
+
+You may also override the default path with a path of your own:
+
+  $module = 'MyTestModule::TestHandler';
+  $url = Apache::TestRequest::module2url($module, {path => '/foo'});
+
+returns:
+
+  http://$hostname:$port/foo
+
+
+
+
+
 =head1 ENVIRONMENT VARIABLES
 
 The following environment variables can affect the behavior of
