@@ -1,8 +1,9 @@
-# Copyright 2001-2004 The Apache Software Foundation
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
@@ -53,6 +54,13 @@ sub new {
 
     $self->{port_counter} = $self->{config}->{vars}->{port};
 
+    $self;
+}
+
+# call this when you already know where httpd is
+sub post_config {
+    my($self) = @_;
+
     $self->{version} = $self->{config}->httpd_version || '';
     $self->{mpm}     = $self->{config}->httpd_mpm     || '';
 
@@ -60,14 +68,8 @@ sub new {
     # string and various variations made by distributions which mangle
     # that string
 
-    # Apache/2.0.50-dev
-    ($self->{rev})   = $self->{version} =~ m|^Apache/(\d)\.|;
-
-    # Apache-AdvancedExtranetServer/1.3.29 (Mandrake Linux/1mdk)
-    ($self->{rev}) ||= $self->{version} =~ m|^Apache.*?/(\d)\.|;
-
-    # IBM_HTTP_SERVER/1.3.19  Apache/1.3.20 (Unix)
-    ($self->{rev}) ||= $self->{version} =~ m|^.*?Apache.*?/(\d)\.|;
+    # Foo-Apache-Bar/x.y.z
+    ($self->{rev}) = $self->{version} =~ m|/(\d)\.|;
 
     if ($self->{rev}) {
         debug "Matched Apache revision $self->{version} $self->{rev}";
@@ -88,6 +90,9 @@ sub new {
 
 sub version_of {
     my($self, $thing) = @_;
+    die "Can't figure out what Apache server generation we are running"
+        unless $self->{rev};
+
     $thing->{$self->{rev}};
 }
 
@@ -95,6 +100,7 @@ my @apache_logs = qw(
 error_log access_log httpd.pid
 apache_runtime_status rewrite_log
 ssl_engine_log ssl_request_log
+cgisock
 );
 
 sub clean {
@@ -112,7 +118,10 @@ sub clean {
 
 sub pid_file {
     my $self = shift;
-    catfile $self->{config}->{vars}->{t_logs}, 'httpd.pid';
+
+    my $vars = $self->{config}->{vars};
+
+    return $vars->{t_pid_file} || catfile $vars->{t_logs}, 'httpd.pid';
 }
 
 sub dversion {
@@ -149,10 +158,25 @@ sub args {
 my %one_process = (1 => '-X', 2 => '-D ONE_PROCESS');
 
 sub start_cmd {
-    my $self = shift;
+    my $self  = shift;
+
+    my $args   = $self->args;
+    my $config = $self->{config};
+    my $vars   = $config->{vars};
+    my $httpd  = $vars->{httpd};
+    my $chroot = '';
+
+    my $one_process = $self->{run}->{opts}->{'one-process'}
+        ? $self->version_of(\%one_process)
+        : '';
+
+    if ($self->{rev} == 1 && !$vars->{chroot}) {
+        $chroot = '-u'
+    }
+
     #XXX: threaded mpm does not respond to SIGTERM with -D ONE_PROCESS
-    my $args = $self->args;
-    return "$self->{config}->{vars}->{httpd} $args";
+
+    return "$httpd $chroot $one_process $args";
 }
 
 sub default_gdbinit {
@@ -250,7 +274,8 @@ sub start_gdb {
         $command = qq{ddd --gdb --debugger "gdb -command $file" $httpd};
     }
     else {
-        $command = "gdb $httpd -command $file";
+        ## defaults to gdb if not set in %ENV or via -debug
+        $command = "$debugger $httpd -command $file";
     }
 
     $self->note_debugging;
@@ -281,14 +306,28 @@ sub start_debugger {
 
     $opts->{debugger} ||= $ENV{MP_DEBUGGER} || 'gdb';
 
-    unless ($debuggers{ $opts->{debugger} }) {
+    # XXX: FreeBSD 5.2+
+    #      gdb 6.1 and before segfaults when trying to
+    #      debug httpd startup code. 6.5 has been proven
+    #      to work.  FreeBSD typically installs this as
+    #      gdb65.
+    #      Is it worth it to check the debugger and os version 
+    #      and die ?
+ 
+    unless (grep { /^$opts->{debugger}/ } keys %debuggers) {
         error "$opts->{debugger} is not a supported debugger",
               "These are the supported debuggers: ".
               join ", ", sort keys %debuggers;
         die("\n");
     }
 
-    my $method = "start_" . $debuggers{ $opts->{debugger} };
+    my $debugger = $opts->{debugger}; 
+    $debugger =~ s/\d+$//;
+
+    my $method = "start_" . $debuggers{$debugger};
+
+    ## $opts->{debugger} is passed through unchanged
+    ## so when we try to run it next, its found.
     $self->$method($opts);
 }
 
